-- ############# General Information ###################
-- ---------
--0 xyz = {}
--1 function alpha()
--2    dostuff with xyz
--3 end
--4
--5 hello = {}
--6 function hello.bravo()
--7   dostuff
--8 end
-- ---------
--
-- The parser will parse each function as well as all text, which
-- comes directly bevore this function.
-- So for function bravo, the parser will use lines 4 to 8. Line 0 will be ignored.
-- The parser will ignore all comments.


-- ############# General Functions #####################

-- Get Resource
if not peakboard then
    peakboard = {}
end
function peakboard.getresource(path)
	return peakboard.getResource(path)
end

if not peakboard then
    peakboard = {}
end
function peakboard.getResource(path)
	return "ts-pbmxdata://resources/" .. path
end

-- Wrapper for the logging method
if not peakboard then
    peakboard = {}
end
function peakboard.log(text)
    log(text)
end

-- Wrapper for old system methods
if not system then
    system = {}
end
function system.shutdown(delay)
    peakboard.shutdown(delay)
end

if not system then
    system = {}
end
function system.restart(delay)
    peakboard.restart(delay)
end

-- ############# Control Functions #####################

--Toogle the visibility of an element
-- Do not use "if not control then.." at this point. It will crash the
-- script evaluator
control = {}
function control.toggle(control)
	if control.Visibility == Visibility.Collapsed then
	   control.Visibility = Visibility.Visible
	else
		control.Visibility = Visibility.Collapsed
	end
end

-- ############# Number Functions #####################
--Returns a number from a string (tostring wrapper with namespace)
function math.tonumber(num)
    return tonumber(num)
end

-- Round without decimals
function math.roundwithoutdecimals(val)
    return math.roundWithoutDecimals(val)
end

function math.roundWithoutDecimals(val)
	return math.floor(val + 0.5)
end

--Rounds a number to a number with n decimals
function math.roundwithdecimals(val, decimals)
    return math.roundWithDecimals(val, decimals)
end

function math.roundWithDecimals(val, decimals)
	local exp = decimals and 10^decimals or 1
	local result = math.ceil(val * exp - 0.5) / exp
	return result;
end

-- ############# String Functions #####################
--Returns a string of a number (tonumber wrapper with namespace)
function string.tostring(str)
    return tostring(str)
end

--Returns true because 'Hello World' starts with 'He'
function string.startswith(val, start)
    return string.startsWith(val, start)
end

function string.startsWith(val, start)
	return string.sub(val,1,string.len(start))==start
end

--Fill up a string with zeros until it has the size of 18 characters
function string.addleadingcharacters(val, leadingChar, fullLenth)
    return string.AddLeadingCharacters(val, leadingChar, fullLenth)
end

function string.AddLeadingCharacters(val, leadingChar, fullLenth)
	return string.rep(leadingChar, fullLenth - string.len(val)) .. val
end

--Fill up a string with zeros until it has the size of 18 characters
function string.addendingcharacters(val, endingChar, fullLenth)
    return string.AddEndingCharacters(val, endingChar, fullLenth)
end

function string.AddEndingCharacters(val, endingChar, fullLenth)
	return val .. string.rep(endingChar, fullLenth - string.len(val))
end

function string.replace(haystack, needle, replacement)
  local buf = {}
  local i = 1
  while i <= #haystack do
    if string.sub(haystack, i, i + #needle - 1) == needle then
      for j = 1, #replacement do
        table.insert(buf, string.sub(replacement, j, j))
      end
      i = i + #needle
    else
      table.insert(buf, string.sub(haystack, i, i))
      i = i + 1
    end
  end
  return table.concat(buf)
end

function string.count(haystack, needle)
  if #needle == 0 then
    return #haystack + 1
  end
  local i = 1
  local count = 0
  while true do
    i = string.find(haystack, needle, i, true)
    if i == nil then
      break
    end
    count = count + 1
    i = i + #needle
  end
  return count
end

function string.firstIndexOf(str, substr)
  local i = string.find(str, substr, 1, true)
  if i == nil then
    return 0
  else
    return i
  end
end

function string.lastIndexOf(str, substr)
  local i = string.find(string.reverse(str), string.reverse(substr), 1, true)
  if i then
    return #str + 2 - i - #substr
  end
  return 0
end

function string.randomLetter(str)
  local index = math.random(string.len(str))
  return string.sub(str, index, index)
end

-- ############# Table Functions #####################

-- Sets the background colour of a row
if not table then
    table = {}
end
function table.colorrow(evnt, colour)
    table.colorRow(evnt, colour)
end

if not table then
    table = {}
end
function table.colorRow(evnt, colour)
	for i = 0, evnt.Columns.Count - 1 do
		evnt.Columns[i].CellControl.Background = colour
	end
end

-- Sets the Text colour of a row
if not table then
    table = {}
end
function table.colorrowtext(evnt, colour)
    table.colorRowText(evnt, colour)
end

if not table then
    table = {}
end
function table.colorRowText(evnt, colour)
	for i = 0, evnt.Columns.Count - 1 do
		evnt.Columns[i].TextControl.Foreground = colour
	end
end

-- Gets the Value of a cell
if not table then
    table = {}
end
function table.getcelltext(evnt, colIndex)
    return table.getCellText(evnt, colIndex)
end

if not table then
    table = {}
end
function table.getCellText(evnt, colIndex)
	return evnt.Columns[colIndex].TextControl.Text
end

-- Sets the value of a cell
if not table then
    table = {}
end
function table.setcelltext(evnt, colIndex, newValue)
    table.setCellText(evnt, colIndex, newValue)
end

if not table then
    table = {}
end
function table.setCellText(evnt, colIndex, newValue)
	evnt.Columns[colIndex].TextControl.Text = newValue
end

-- ########## Date Functions #############

if not date then
    date = {}
end
function date.parsedatedmy(timeString)
	local pattern = "(%d+).(%d+).(%d+)"
	local nday, nmonth, nyear = timeString:match(pattern)
	return os.time{year = nyear, month = nmonth, day = nday}
end

if not date then
    date = {}
end
function date.parsedateymd(timeString)
	local pattern = "(%d+).(%d+).(%d+)"
	local nyear, nmonth, nday = timeString:match(pattern)
	return os.time{year = nyear, month = nmonth, day = nday}
end

if not date then
    date = {}
end
function date.parsedate(timeString, pattern)
	local nday, nmonth, nyear, nhour, nmin, nsec = timeString:match(pattern)
	return os.time{year = nyear, month = nmonth, day = nday, hour = nhour, min = nmin, sec = nsec}
end

if not date then
    date = {}
end
function date.addyear(date, var)
	peakboard.log(date .. '   ' .. var)
	local ndate = date + (var*365*24*60*60)
	peakboard.log(ndate)
	peakboard.log('returned')
	return ndate
end

if not date then
    date = {}
end
function date.addmonth(date, var)
    local newdate = os.date("%d.%m.%Y.%H.%M.%S", date)
   local nday, nmonth, nyear, nhour, nmin, nsec = string.match(newdate, '(%d+).(%d+).(%d+).(%d+).(%d+).(%d+)')
   nyear = nyear + math.floor((nmonth + var)/12)
   nmonth = (nmonth + var) % 12
   nday = tonumber(nday)
    if (nmonth == 4 or nmonth == 6 or nmonth == 9 or nmonth == 11) and nday > 30 then
        -- Apr, Jun, Sep, Nov can have at most 30 days
        nday = 30
    elseif nmonth == 2 then
        -- Feb
        if (nyear%400 == 0 or (nyear%100 ~= 0 and nyear%4 == 0)) and nday > 29 then
            -- if leap year, days can be at most 29
            nday = 29
        elseif nday > 28 then
            -- else 28 days is the max
            nday = 28
        end
    elseif nday > 31 then
        nday = 31
    end
   return os.time{year = nyear, month = nmonth, day = nday, hour = nhour, min = nmin, sec = nsec}
end

if not date then
    date = {}
end
function date.addday(date, var)
	local ndate = date + (var*24*60*60)
	return ndate
end

if not date then
    date = {}
end
function date.addhour(date, var)
	local ndate = date + (var*60*60)
	return ndate
end

if not date then
    date = {}
end
function date.addminute(date, var)
	local ndate = date + (var*60)
	return ndate
end

if not date then
    date = {}
end
function date.addsecond(date, var)
	local ndate = date + (var)
	return ndate
end

if not date then
    date = {}
end
function date.format(date, pattern)
    return os.date(pattern, date)
end

if not date then
    date = {}
end
function date.create(nyear, nmonth, nday)
    return os.time{year=nyear, month=nmonth, day=nday, hour=nhour, min=nmin, sec=nsec}
end

if not date then
    date = {}
end
function date.create(nyear, nmonth, nday, nhour, nmin, nsec)
    return os.time{year=nyear, month=nmonth, day=nday, hour=nhour, min=nmin, sec=nsec}
end

if not date then
    date = {}
end
function date.difference(date1, date2)
    return os.difftime (date1, date2)
end

function private_getyearbegindayofweek(tm)
    local yearBegin = os.time{year=os.date("*t",tm).year,month=1,day=1}
    local yearBeginDayOfWeek = tonumber(os.date("%w",yearBegin))
    -- sunday correct from 0 -> 7
    if(yearBeginDayOfWeek == 0) then yearBeginDayOfWeek = 7 end
    return yearBeginDayOfWeek
end

function private_getdayadd(tm)
    local yearBeginDayOfWeek = private_getyearbegindayofweek(tm)
    if(yearBeginDayOfWeek < 5 ) then
        -- first day is week 1
        return (yearBeginDayOfWeek - 2)
    else
        -- first day is week 52 or 53
        return (yearBeginDayOfWeek - 9)
    end
end

if not date then
    date = {}
end
function date.getweeknumberofyear(tm)
    local dayOfYear = os.date("%j",tm)
    local dayAdd = private_getdayadd(tm)
    local dayOfYearCorrected = dayOfYear + dayAdd
    if(dayOfYearCorrected < 0) then
        -- week of last year - decide if 52 or 53
        local lastYearBegin = os.time{year=os.date("*t",tm).year-1,month=1,day=1}
        local lastYearEnd = os.time{year=os.date("*t",tm).year-1,month=12,day=31}
        dayAdd = private_getdayadd(lastYearBegin)
        dayOfYear = dayOfYear + os.date("%j",lastYearEnd)
        dayOfYearCorrected = dayOfYear + dayAdd
    end
    local weekNum = math.floor((dayOfYearCorrected) / 7) + 1
    if((dayOfYearCorrected > 0) and weekNum == 53) then
        -- check if it is not considered as part of week 1 of next year
        local nextYearBegin = os.time{year=os.date("*t",tm).year+1,month=1,day=1}
        local yearBeginDayOfWeek = private_getyearbegindayofweek(nextYearBegin)
        if(yearBeginDayOfWeek < 5 ) then
            weekNum = 1
        end
    end
    return weekNum
end

-- ########## Peakboard Functions #############

if not peakboard then
    peakboard = {}
end
function peakboard.pushdata(receiver, user, password, visualization, datakey, newvalue)
	local cookieContainer = http.createCookieContainer();
	local auth = http.createBasicAuth(user, password)
	local headers = { { name = 'Content-Type', value = 'application/csv' }, {name = 'Authorization', value=auth } }
	local result = http.post('http://' .. receiver ..  ':40404/api/datapush?id=' .. visualization .. '&datakey=' .. datakey, headers, newvalue, cookieContainer)
end